import tkinter as tk
from tkinter import ttk
from tkinter import *
from player_data import FIFA22playerData
import matplotlib.pyplot as plt
from matplotlib.backends.backend_tkagg import FigureCanvasTkAgg
import numpy as np
import pandas as pd


class Application(tk.Tk):

    def __init__(self, data: FIFA22playerData):
        super().__init__()
        self.data = data.player_data
        self.player = data.get_all_name()
        self.stat = data.player_stat
        self.title('FIFA 22 Data Analysis')
        self.geometry('1400x600')
        self.displayGraph_frame()
        self.menu_frame()
        self.quit_button()

    def displayGraph_frame(self):
        ''' this frame used to show the graphical result'''
        self.frame1 = ttk.LabelFrame(self, text='Result')
        self.frame1.pack(fill="both", expand=True, side="top")

    def menu_frame(self):
        ''' this frame used to selected the button which user would like to use'''
        # create frame
        self.frame2 = ttk.LabelFrame(self, text='Menu')
        self.frame2.pack(fill="both", side="bottom")

        # create plot button
        # - if user click each button, display frame(frame1) will display the each graph
        self.ovr_plt_btn = Button(self.frame2, text='Overall distribution',
                                  command=self.overalldist_graph)
        self.ovr_plt_btn.pack(side='left')

        self.h_plt_btn = Button(self.frame2, text='Height distribution',
                                command=self.heightdist_graph)
        self.h_plt_btn.pack(side='left')

        self.pie_plt_btn = Button(self.frame2,
                                  text='Portion of preferred foot',
                                  command=self.footportion_graph)
        self.pie_plt_btn.pack(side='left')

        self.ts_plt_btn = Button(self.frame2,
                                 text='Timeseries graph',
                                 command=self.timeserie_graph)
        self.ts_plt_btn.pack(side='left')

        self.corr_plt_btn = Button(self.frame2,
                                   text='Correlation between strength and skill dribbling',
                                   command=self.scatter_graph)
        self.corr_plt_btn.pack(side='left')

        self.ovr_dst_btn = Button(self.frame2,
                              text='Overall descriptive statistic',
                              command=self.dst_ovr)
        self.ovr_dst_btn.pack(side='left')

        self.h_dst_btn = Button(self.frame2,
                              text='Height descriptive statistic',
                              command=self.dst_h)
        self.h_dst_btn.pack(side='left')

    def overalldist_graph(self):
        ''' create the distribution of overall rating graph'''
        self.clear_frame()
        ovr_data = self.data['overall']
        ovr = np.arange(47, 94)
        ovr_freq = ovr_data.value_counts().sort_index()
        fig = plt.Figure()
        ax1 = fig.add_axes([0.1, 0.1, 0.8, 0.8])
        ax1.bar(ovr, ovr_freq, color='blue')
        ax1.set_xlabel('Overall rating')
        ax1.set_ylabel('Frequency')
        ax1.set_title('Overall rating distribution')
        canvas = FigureCanvasTkAgg(fig, self.frame1)
        canvas.draw()
        canvas.get_tk_widget().pack(fill=tk.BOTH, expand=True)

    def heightdist_graph(self):
        ''' create the distribution of height graph'''
        self.clear_frame()
        height_data = self.data['height_cm']
        height = np.arange(155, 207)
        height = height[(height != 205) & (height != 204) & (height != 157)]
        height_freq = height_data.value_counts().sort_index()
        fig = plt.Figure()
        ax2 = fig.add_axes([0.1, 0.1, 0.8, 0.8])
        ax2.bar(height, height_freq, color='green')
        ax2.set_xlabel('Height')
        ax2.set_ylabel('Frequency')
        ax2.set_title('Height distribution')
        canvas = FigureCanvasTkAgg(fig, self.frame1)
        canvas.draw()
        canvas.get_tk_widget().pack(fill=tk.BOTH, expand=True)

    def footportion_graph(self):
        ''' create the portion of preferred foot graph '''
        self.clear_frame()
        foot_data = self.data['preferred_foot']
        foot_freq = foot_data.value_counts().sort_index()
        fig = plt.figure()
        ax = fig.add_axes([0.1, 0.1, 0.8, 0.8])
        ax.pie(foot_freq, labels=['Left foot', 'Right foot'],
               autopct='%1.1f%%', startangle=90)
        ax.axis('equal')
        ax.set_title('Portion of preferred foot')
        canvas = FigureCanvasTkAgg(fig, self.frame1)
        canvas.draw()
        canvas.get_tk_widget().pack(fill=tk.BOTH, expand=True)

    def timeserie_graph(self):
        self.clear_frame()
        self.data['y'] = pd.to_datetime(self.data['dob']).dt.strftime('%Y')
        byear = self.data['y']
        byear_freq = byear.value_counts().sort_index()
        year = np.arange(1967, 2005)
        year = year[
            (year != 1977) & (year != 1976) & (year != 1975) & (year != 1974)
            & (year != 1973) & (year != 1972) & (year != 1971) & (
                    year != 1970) & (year != 1969) & (year != 1968)]
        fig = plt.figure()
        ax = fig.add_axes([0.1, 0.1, 0.8, 0.8])
        line1, = ax.plot(year, byear_freq)
        line1.set_color('r')
        line1.set_marker('o')
        line1.set_linewidth(1)
        ax.set_xlabel('Birth year')
        ax.set_ylabel('Frequency')
        ax.set_title('Number of players that birth in each year')
        canvas = FigureCanvasTkAgg(fig, self.frame1)
        canvas.draw()
        canvas.get_tk_widget().pack(fill=tk.BOTH, expand=True)

    def scatter_graph(self):
        self.clear_frame()
        str = self.data['power_strength']
        drib = self.data['skill_dribbling']
        fig = plt.figure()
        plt.scatter(str, drib, c='blue')
        plt.xlabel('Power strength')
        plt.ylabel('Skill dribbling')
        plt.title('Relationship between strength and skill dribbling')
        canvas = FigureCanvasTkAgg(fig, self.frame1)
        canvas.draw()
        canvas.get_tk_widget().pack(fill=tk.BOTH, expand=True)

    def dst_ovr(self):
        self.clear_frame()
        ovr_mean = self.data['overall'].mean()
        ovr_median = self.data['overall'].median()
        ovr_mode = self.data['overall'].mode().loc[0,]
        ovr_std = self.data['overall'].std()
        ovr_q1 = self.data['overall'].quantile(0.25)
        ovr_q3 = self.data['overall'].quantile(0.75)
        title = Label(self.frame1, text='Descriptive statistic of overall rating', font=('Ariel',30))
        mean_label = Label(self.frame1, text=f'Mean: {ovr_mean:.2f}', font=('Ariel',20))
        median_label = Label(self.frame1, text=f'Median: {ovr_median:.2f}', font=('Ariel',20))
        mode_label = Label(self.frame1, text=f'Mode: {ovr_mode:.2f}', font=('Ariel',20))
        std_label = Label(self.frame1, text=f'Standard deviation: {ovr_std:.2f}', font=('Ariel',20))
        q1_label = Label(self.frame1, text=f'1st-quartile: {ovr_q1:.2f}', font=('Ariel',20))
        q3_label = Label(self.frame1, text=f'3rd-quartile: {ovr_q3:.2f}', font=('Ariel',20))
        title.pack()
        mean_label.pack()
        median_label.pack()
        mode_label.pack()
        std_label.pack()
        q1_label.pack()
        q3_label.pack()

    def dst_h(self):
        self.clear_frame()
        ovr_mean = self.data['height_cm'].mean()
        ovr_median = self.data['height_cm'].median()
        ovr_mode = self.data['height_cm'].mode().loc[0,]
        ovr_std = self.data['height_cm'].std()
        ovr_q1 = self.data['height_cm'].quantile(0.25)
        ovr_q3 = self.data['height_cm'].quantile(0.75)
        title = Label(self.frame1, text='Descriptive statistic of overall rating', font=('Ariel',30))
        mean_label = Label(self.frame1, text=f'Mean: {ovr_mean:.2f}', font=('Ariel',20))
        median_label = Label(self.frame1, text=f'Median: {ovr_median:.2f}', font=('Ariel',20))
        mode_label = Label(self.frame1, text=f'Mode: {ovr_mode:.2f}', font=('Ariel',20))
        std_label = Label(self.frame1, text=f'Standard deviation: {ovr_std:.2f}', font=('Ariel',20))
        q1_label = Label(self.frame1, text=f'1st-quartile: {ovr_q1:.2f}', font=('Ariel',20))
        q3_label = Label(self.frame1, text=f'3rd-quartile: {ovr_q3:.2f}', font=('Ariel',20))
        title.pack()
        mean_label.pack()
        median_label.pack()
        mode_label.pack()
        std_label.pack()
        q1_label.pack()
        q3_label.pack()

    def quit_button(self):
        self.quit_btn = Button(text='Quit', command=self.destroy)
        self.quit_btn.pack(side='bottom')

    def clear_frame(self):
        for graph in self.frame1.winfo_children():
            graph.destroy()

    def run(self):
        self.mainloop()
